local mapHelper = {
    dependencies = { "utils", "list", "dictionary" },
    cellArray = {}
}

mapHelper.gatherElements = {
    ["1"] = "Frêne",
    ["8"] = "Chêne",
    ["17"] = "Fer",
    ["24"] = "Argent",
    ["25"] = "Or",
    ["26"] = "Pierre de Bauxite",
    ["28"] = "If",
    ["29"] = "Ebène",
    ["30"] = "Orme",
    ["31"] = "Erable",
    ["32"] = "Charme",
    ["33"] = "Châtaignier",
    ["34"] = "Noyer",
    ["35"] = "Merisier",
    ["37"] = "Pierre de Kobalte",
    ["38"] = "Blé",
    ["39"] = "Houblon",
    ["42"] = "Lin",
    ["43"] = "Orge",
    ["44"] = "Seigle",
    ["45"] = "Avoine",
    ["46"] = "Chanvre",
    ["47"] = "Malt",
    ["52"] = "Etain",
    ["53"] = "Pierre Cuivrée",
    ["54"] = "Manganèse",
    ["55"] = "Bronze",
    ["61"] = "Edelweiss",
    ["66"] = "Menthe Sauvage",
    ["67"] = "Trèfle à 5 feuilles",
    ["68"] = "Orchidée Freyesque",
    ["71"] = "Petits poissons (mer)",
    ["72"] = "Somoon Agressif",
    ["73"] = "Pwoulpe",
    ["74"] = "Poissons (rivière)",
    ["75"] = "Petits poissons (rivière)",
    ["76"] = "Gros poissons (rivière)",
    ["77"] = "Poissons (mer)",
    ["78"] = "Gros poissons (mer)",
    ["79"] = "Poissons géants (rivière)",
    ["80"] = "Truite Vaseuse",
    ["81"] = "Poissons géants (mer)",
    ["84"] = "Puits",
    ["98"] = "Bombu",
    ["100"] = "Pichon",
    ["101"] = "Oliviolet",
    ["108"] = "Bambou",
    ["109"] = "Bambou sombre",
    ["110"] = "Bambou sacré",
    ["111"] = "Riz",
    ["112"] = "Pandouille",
    ["113"] = "Dolomite",
    ["114"] = "Silicate",
    ["121"] = "Kaliptus",
    ["131"] = "Perce-neige",
    ["132"] = "Poisson de Frigost",
    ["133"] = "Tremble",
    ["134"] = "Frostiz",
    ["135"] = "Obsidienne",
    ["169"] = "Bar Akouda",
    ["221"] = "Fleur de Sutol",
    ["225"] = "Piraniak"
}

function mapHelper:init()
    self.directions = self.list()
    local tmp = self.gatherElements
    self.gatherElements = self.dictionary():fromTable(tmp)

    self.directions:add("left")
    self.directions:add("right")
    self.directions:add("top")
    self.directions:add("bottom")
    self:initCellsArray()
end

------------------- Cells helper -------------------

function mapHelper:initCellsArray()
    local width = 14
    local height = 40
    for cellId = 0, width * height - 1 do
        local x = cellId % width
        local y = math.floor(cellId / width)
        self.cellArray[cellId] = {x = x, y = y}
    end
end

function mapHelper:getWalkableZones()
    local movableCells = map:getWalkableCells(false)
    local visited = {}
    local zones = {}

    for _, cell in pairs(movableCells) do
        if not visited[cell] then
            local zone = self:dfs(cell, visited, movableCells)
            table.insert(zones, zone)
        end
    end
    return zones
end

function mapHelper:getWalkableZoneByCellId(cellId)
    local movableCells = map:getWalkableCells(false)
    local visited = {}
    local zone = self:dfs(cellId, visited, movableCells)
    return zone
end

function mapHelper:dfs(cell, visited, movableCells)
    local stack = {cell}
    local zone = {}

    while #stack > 0 do
        local current = table.remove(stack)

        if not visited[current] then
            visited[current] = true
            table.insert(zone, current)

            local adjacentCells = self:getAdjacentCells(current)
            --self.logger:log("adjacent cells: ")
            --self.logger:log(adjacentCells)
            for _, adjacentCell in pairs(adjacentCells) do
                if self.utils:isInTable(movableCells, adjacentCell) and not visited[adjacentCell] then
                    if self:isDiagonalWalkable(current, movableCells) then
                        --self.logger:log("current: " .. current)
                        --self.logger:log("adding: " .. adjacentCell)
                        table.insert(stack, adjacentCell)
                    end
                end
            end
        end
    end

    return zone
end

function mapHelper:isDiagonalWalkable(cell, cellList)
    local walkableCells = cellList
    local adjCell1 = self:getAdjacentCells(cell)
    local possibleDir = self:possibleDirection(cell)
    local coord = self:cellIdToCoord(cell)
    local isPairs = coord.y % 2 == 0

    -- Vérifiez si les cellules diagonales sont walkables
    if not possibleDir then
        for _, adjCell in pairs(adjCell1) do
            if self.utils:isInTable(walkableCells, adjCell) then
                return true
            end
        end
        return false

    elseif possibleDir == "left" then
        if isPairs then
            return self.utils:isInTable(walkableCells, adjCell1["bg"]) or self.utils:isInTable(walkableCells, adjCell1["bd"])
        else
            return self.utils:isInTable(walkableCells, adjCell1["hg"]) or self.utils:isInTable(walkableCells, adjCell1["hd"])
        end
    elseif possibleDir == "right" then
        if isPairs then
            return self.utils:isInTable(walkableCells, adjCell1["hg"]) or self.utils:isInTable(walkableCells, adjCell1["hd"])
        else
            return self.utils:isInTable(walkableCells, adjCell1["bg"]) or self.utils:isInTable(walkableCells, adjCell1["bd"])
        end

    elseif possibleDir == "top" then
        if not isPairs then
            return self.utils:isInTable(walkableCells, adjCell1["hg"]) or self.utils:isInTable(walkableCells, adjCell1["hd"])
        else
            return self.utils:isInTable(walkableCells, adjCell1["bg"]) or self.utils:isInTable(walkableCells, adjCell1["bd"])
        end
    elseif possibleDir == "bottom" then
        if isPairs then
            return self.utils:isInTable(walkableCells, adjCell1["hg"]) or self.utils:isInTable(walkableCells, adjCell1["hd"])
        else
            return self.utils:isInTable(walkableCells, adjCell1["bg"]) or self.utils:isInTable(walkableCells, adjCell1["bd"])
        end
    end
end

function mapHelper:getAdjacentCells(cellId)
    local coord = self:cellIdToCoord(cellId)
    local adjacentOffsets
    if coord.y % 2 == 0 then
        --self.logger:debug("pairs")
        adjacentOffsets = {
            ["g"] = {x = -1, y =  0},  -- Gauche
            ["d"] = {x =  1, y =  0},  -- Droite
            ["h"] = {x =  0, y = -2},  -- Haut
            ["b"] = {x = 0, y = 2},   -- Bas
            ["hg"] = {x =  -1, y = -1},  -- Haut gauche
            ["bg"] = {x =  -1, y = 1},  -- Bas gauche
            ["hd"] = {x =  0, y = -1},  -- Haut droite
            ["bd"] = {x = 0, y = 1}   -- Bas droit
        }
    else
        --self.logger:debug("impairs")
        adjacentOffsets = {
            ["g"] = {x = -1, y =  0},  -- Gauche
            ["d"] = {x =  1, y =  0},  -- Droite
            ["h"] = {x =  0, y = -2},  -- Haut
            ["b"] = {x = 0, y = 2},   -- Bas
            ["hg"] = {x =  0, y = -1},  -- Haut gauche
            ["bg"] = {x =  0, y = 1},  -- Bas gauche
            ["hd"] = {x =  1, y = -1},  -- Haut droite
            ["bd"] = {x = 1, y = 1}   -- Bas droit
        }
    end

    local adjacentCells = {}
    for k, offset in pairs(adjacentOffsets) do
        local adjCoord = {x = coord.x + offset.x, y = coord.y + offset.y}
        if self:isCoordValid(adjCoord) then
            local adjCellId = adjCoord.y * 14 + adjCoord.x
            adjacentCells[k] = adjCellId
        end
    end

    return adjacentCells
end

function mapHelper:cellsBFSUntil(cell, targetCells)
    local visited = {}
    local queue = {cell}

    while #queue > 0 do
        local current = table.remove(queue, 1)
        if not visited[current] then
            visited[current] = true

            if self.utils:isInTable(targetCells, current) then
                return current  -- Found a target cell, return it
            end

            local adjacentCells = self:getAdjacentCells(current)

            for _, adjacentCell in pairs(adjacentCells) do
                if not visited[adjacentCell] then
                    table.insert(queue, adjacentCell)
                end
            end
        end
    end

    return nil  -- No target cells were found
end

function mapHelper:cellIdToCoord(cellId)
    if self:isCellIdValid(cellId) then
        return self.cellArray[cellId]
    end

    return nil
end

function mapHelper:coordToCellId(coord)
    if self:isCoordValid(coord) then
        return coord.y * 14 + coord.x + 1
    end
end

function mapHelper:isCellIdValid(cellId)
	return (cellId >= 0 and cellId < 560)
end

function mapHelper:isCoordValid(coord)
    local width = 14
    local height = 40
    --return true
    return coord.x >= 0 and coord.y >= 0 and coord.x < width and coord.y < height
end

------------------- Get direction helpers -------------------

function mapHelper:getZoneDirectionsFormatted(zone)
    local directions = {}
    local result = {}

    --self.logger:log(self:cellIdToCoord(14))
    for _, cellId in pairs(zone) do
        local coord = self:cellIdToCoord(cellId)
        --self.logger:log(coord)
        -- Si la cellule est sur le bord supérieur, notez que nous pouvons nous déplacer vers le haut
        if not (coord.x == 0 and coord.y < 2) and not (coord.x == 13 and coord.y == 1) and (coord.y == 0 or coord.y == 1) and not directions["top"] then
            --self.logger:log(coord)
            --self.logger:log("top")
            if result["top"] == nil then
                result["top"] = {}
            end

            table.insert(result["top"], "top(" .. cellId .. ")")
            --directions["top"] = true
        end
        -- Si la cellule est sur le bord inférieur, notez que nous pouvons nous déplacer vers le bas
        if not (coord.x == 13 and coord.y >= 38) and not (coord.x == 0 and coord.y == 38) and (coord.y == 38 or coord.y == 39) and not directions["bottom"] then
            --self.logger:log(coord)
            --self.logger:log("bottom")
            if result["bottom"] == nil then
                result["bottom"] = {}
            end

            table.insert(result["bottom"], "bottom(" .. cellId .. ")")
            --directions["bottom"] = true
        end
        -- Si la cellule est sur le bord gauche, notez que nous pouvons nous déplacer vers la gauche
        if (coord.x == 0 and coord.y < 39) and not directions["left"] then
            --self.logger:log(coord)
            --self.logger:log("left")
            if result["left"] == nil then
                result["left"] = {}
            end

            table.insert(result["left"], "left(" .. cellId .. ")")
            --directions["left"] = true
        end
        -- Si la cellule est sur le bord droit, notez que nous pouvons nous déplacer vers la droite
        if (coord.x == 13 and coord.y > 0) and not directions["right"] then
            --self.logger:log(coord)
            --self.logger:log("right")
            if result["right"] == nil then
                result["right"] = {}
            end

            table.insert(result["right"], "right(" .. cellId .. ")")
            --directions["right"] = true
        end
    end

    return result
end

function mapHelper:getOppositeDirection(dir)
    if dir == "left" then
        return "right"
    elseif dir == "right" then
        return "left"
    elseif dir == "top" then
        return "bottom"
    elseif dir == "bottom" then
        return "top"
    end
end

function mapHelper:getCellDirection(cellId)
    local coord = self:cellIdToCoord(cellId)
    local directions = {}

    -- If the cell is on the top row of the map, note that it's in the 'top' direction
    if coord.y <= 20 then
        directions["top"] = true
    else
        directions["bottom"] = true
    end

    -- If the cell is on the left column of the map, note that it's in the 'left' direction
    if coord.x <= 7 then
        directions["left"] = true
    else
        directions["right"] = true
    end

    return directions
end

function mapHelper:possibleDirection(cellId)
    if cellId then
        local coord = self:cellIdToCoord(cellId)
        --self.logger:log(cellId)
        --self.logger:log(coord)
        -- If the cell is on the top row of the map, note that it's in the 'top' direction
        if not (coord.x == 0 and coord.y < 2) and not (coord.x == 13 and coord.y == 1) and (coord.y == 0 or coord.y == 1) then
            return "top"
        elseif not (coord.x == 13 and coord.y >= 38) and not (coord.x == 0 and coord.y == 38) and (coord.y == 38 or coord.y == 39) then
            return "bottom"
        end

        -- If the cell is on the left column of the map, note that it's in the 'left' direction
        if coord.x == 0 and coord.y < 39 then
            return "left"
        elseif coord.x == 13 and coord.y > 0 then
            return "right"
        end
    end
end

function mapHelper:getMirrorDirection(directionCell)
    local direction, cellId = directionCell:match("^(%a+)%((%d+)%)$")
    if cellId then
        cellId = tonumber(cellId)
        local coord = self:cellIdToCoord(cellId)
        local mirrorCoord = {x = coord.x, y = coord.y}
    
        if direction == "left" then
            mirrorCoord.x = 13
        elseif direction == "right" then
            mirrorCoord.x = 0
        elseif direction == "top" then
            mirrorCoord.y = coord.y + 38
        elseif direction == "bottom" then
            mirrorCoord.y = coord.y - 38
        else
            return nil -- Direction invalide
        end
    
        local mirrorCellId = mirrorCoord.y * 14 + mirrorCoord.x
        local mirrorDirection = self:getOppositeDirection(direction)
        return mirrorDirection .. "(" .. mirrorCellId .. ")"
    end
end

function mapHelper:getZoneDirections(zone)
    local directions = {}

    for _, cellId in pairs(zone) do
        local coord = self:CellIdToCoord(cellId)

        -- If the cell is on the top row of the map, note that we can move up
        if not (coord.x == 13 and coord.y == 1) and coord.y == 0 or coord.y == 1 then
            directions["top"] = true
        elseif not (coord.x == 0 and coord.y == 38) and coord.y == 38 or coord.y == 39 then
            directions["bottom"] = true
        end

        -- If the cell is on the left column of the map, note that it's in the 'left' direction
        if coord.x == 0 and coord.y < 39 then
            directions["left"] = true
        elseif coord.x == 13 and coord.y > 0 then
            directions["right"] = true
        end
    end

    return directions
end

------------------- Get Gather Elements -------------------

function mapHelper:getGatherElements()
    local m = map:getMap()
    local ret = {}
    for _, interactive in pairs(m.interactiveElements) do
        local elementName = self.gatherElements:get(tostring(interactive.elementTypeId))
        if elementName then
            if ret[elementName] == nil then
                ret[elementName] = {
                    count = 1,
                    typeId = interactive.elementTypeId
                }
            else
                ret[elementName].count = ret[elementName].count + 1
            end
        end
    end
    return ret
end

function mapHelper:getGatherElementsByZone()
    local walkableZones = self:getWalkableZones()
    local m = map:getMap()
    local ret = {}
    for _, interactive in pairs(m.interactiveElements) do
        local elementName = self.gatherElements:get(tostring(interactive.elementTypeId))
        if elementName then
            for _, statedElement in pairs(m.statedElements) do
                if interactive.elementId == statedElement.elementId then
                    if ret[elementName] == nil then
                        ret[elementName] = {
                            count = 1,
                            typeId = interactive.elementTypeId,
                            cells = {statedElement.elementCellId}
                        }
                    else
                        ret[elementName].count = ret[elementName].count + 1
                        table.insert(ret[elementName].cells, statedElement.elementCellId)
                    end
                    break
                end
            end
        end
    end

    local countInteractive = {}
    for i, zone in pairs(walkableZones) do
        for _, interactiveElement in pairs(ret) do
            for _, interactiveCell in pairs(interactiveElement.cells) do
                if self:cellsBFSUntil(interactiveCell, zone) then
                    local elementName = self.gatherElements:get(tostring(interactiveElement.typeId))
                    if countInteractive[i] == nil then
                        countInteractive[i] = {}
                    end
                    if countInteractive[i][elementName] == nil then
                        countInteractive[i][elementName] = { name = elementName, typeId = interactiveElement.typeId, count = 1 }
                    else
                        countInteractive[i][elementName].count = countInteractive[i][elementName].count + 1
                    end

                end
            end
        end
    end

    return countInteractive
end

function mapHelper:getGatherElementsZoneByCell(cellId)
    local walkableZones = self:getWalkableZones()
    local m = map:getMap()
    local ret = {}
    for _, interactive in pairs(m.interactiveElements) do
        local elementName = self.gatherElements:get(tostring(interactive.elementTypeId))
        if elementName then
            for _, statedElement in pairs(m.statedElements) do
                if interactive.elementId == statedElement.elementId then
                    if ret[elementName] == nil then
                        ret[elementName] = {
                            count = 1,
                            typeId = interactive.elementTypeId,
                            cells = {statedElement.elementCellId}
                        }
                    else
                        ret[elementName].count = ret[elementName].count + 1
                        table.insert(ret[elementName].cells, statedElement.elementCellId)
                    end
                    break
                end
            end
        end
    end

    local countInteractive = {}
    for i, zone in pairs(walkableZones) do
        if self:cellsBFSUntil(cellId, zone) then
            for _, interactiveElement in pairs(ret) do
                for _, interactiveCell in pairs(interactiveElement.cells) do
                    if self:cellsBFSUntil(interactiveCell, zone) then
                        local elementName = self.gatherElements:get(tostring(interactiveElement.typeId))
                        if countInteractive[elementName] == nil then
                            countInteractive[elementName] = { name = elementName, typeId = interactiveElement.typeId, count = 1 }
                        else
                            countInteractive[elementName].count = countInteractive[elementName].count + 1
                        end

                    end
                end
            end
        end
    end

    return countInteractive
end

------------------- Detect multiple directions -------------------

function mapHelper:sortCellsByDirection(cells, direction)
    local coords = {}

    for _, cell in ipairs(cells) do
        local coord = self:cellIdToCoord(cell)
        table.insert(coords, {cell = cell, x = coord.x, y = coord.y})
    end

    table.sort(coords, function(a, b)
        if direction == "top" or direction == "bottom" then
            if a.y == b.y then
                return a.x < b.x
            else
                if direction == "top" then
                    return (a.y < b.y) and (a.x < b.x)
                else
                    return a.y > b.y
                end
            end
        elseif direction == "left" or direction == "right" then
            if a.x == b.x then
                return a.y < b.y
            else
                if direction == "left" then
                    return a.x < b.x
                else
                    return a.x > b.x
                end
            end
        end
    end)

    local sortedCells = {}
    for _, coord in ipairs(coords) do
        table.insert(sortedCells, coord.cell)
    end


    if direction == "bottom" then
        --self.logger:log(sortedCells)
        --global:finishScript()
    end

    return sortedCells
end

function mapHelper:detectMultipleDirections(movableCells)
    --local movableCells = map:getWalkableCells(false)
    local results = {}
   -- self.logger:log(movableCells)
    --global:finishScript()
    for _, direction in ipairs({"left", "right", "top", "bottom"}) do
        local segments = self:detectSegments(movableCells, direction)
        for _, segment in ipairs(segments) do
            if #segment > 0 then
                results[direction] = results[direction] or {}
                local segmentString = table.concat(segment, ", ")
                table.insert(results[direction], direction .. "(" .. segmentString .. ")")
            end
        end
    end

    return results
end

function mapHelper:detectSegments(movableCells, direction)
    local sortedCells = self:sortCellsByDirection(movableCells, direction)
    local segments = {}
    local visited = {}

    for _, cell in ipairs(sortedCells) do
        if not visited[cell] and self:isCellOnEdge(cell, direction) then
            local segment = self:dfsForSegment(cell, visited, sortedCells, direction)
            if #segment > 0 then
                table.insert(segments, segment)
            end
        end
    end

    -- Fusionner les segments adjacents
    local mergedSegments = self:mergeSegments(segments, direction)
    return mergedSegments
end

function mapHelper:dfsForSegment(cell, visited, movableCells, direction)
    local stack = {cell}
    local segment = {}
    local DIRS = {
        left = {"g", "hg", "bg"},
        right = {"d", "hd", "bd"},
        bottom = {"b", "bg", "bd"},
        top = {"h", "hg", "hd"}
    }

    while #stack > 0 do
        local current = table.remove(stack)
        if not visited[current] then
            visited[current] = true
            table.insert(segment, current)

            local adjacentCells = self:getAdjacentCells(current)
            for _, dir in ipairs(DIRS[direction]) do
                local nextCell = adjacentCells[dir]
                if nextCell and self.utils:isInTable(movableCells, nextCell) and not visited[nextCell] then
                    table.insert(stack, nextCell)
                end
            end
        end
    end

    -- Trier le segment pour garantir l'ordre correct
    table.sort(segment)
    return segment
end

function mapHelper:mergeSegments(segments, direction)
    local mergedSegments = {}
    local segmentMerged = {}

    local function areAnyCellsConnected(segment1, segment2)
        -- Vérification améliorée des cellules adjacentes dans toutes les directions pertinentes
        for _, cell1 in ipairs(segment1) do
            local adjacentCells = self:getAdjacentCells(cell1)
            for _, cell2 in ipairs(segment2) do
                if self.utils:isInTable(adjacentCells, cell2) then
                    return true
                end
            end
        end
        return false
    end

    local function mergeConnectedSegments(currentSegment, segments)
        for i = 1, #segments do
            if not segmentMerged[i] and areAnyCellsConnected(currentSegment, segments[i]) then
                for _, cell in ipairs(segments[i]) do
                    table.insert(currentSegment, cell)
                end
                segmentMerged[i] = true
                mergeConnectedSegments(currentSegment, segments)
            end
        end
    end
    -- Première passe pour fusionner les segments directement connectés
    for i, segment in ipairs(segments) do
        if not segmentMerged[i] then
            local currentSegment = {table.unpack(segment)}
            segmentMerged[i] = true
            mergeConnectedSegments(currentSegment, segments)
            table.insert(mergedSegments, currentSegment)
        end
    end

    -- Vérification améliorée et fusion des segments dans toutes les directions
    local changes = true
    while changes do
        changes = false
        for i = 1, #mergedSegments do
            for j = i + 1, #mergedSegments do
                if areAnyCellsConnected(mergedSegments[i], mergedSegments[j]) then
                    -- Fusionner les deux segments
                    for _, cell in ipairs(mergedSegments[j]) do
                        table.insert(mergedSegments[i], cell)
                    end
                    -- Supprimer le segment fusionné
                    table.remove(mergedSegments, j)
                    changes = true
                    break -- Recommencer le processus car la liste a changé
                end
            end
            if changes then break end -- Redémarrer si des changements ont été effectués
        end
    end

    -- Tri des segments après fusion
    for i, segment in ipairs(mergedSegments) do
        table.sort(segment) -- On trie pour avoir les segments dans un ordre logique
    end

    return mergedSegments
end

function mapHelper:getAdjacentCellsForDirection(cellId, direction)
    local adjacentCells = self:getAdjacentCells(cellId)
    local coord = self:cellIdToCoord(cellId)
    local directionOffsets = {
        --left = {"bd", "bg"},
        --right = {"bd", "bg"},
        --top = {"bd","hd"},
        --bottom = {"bd", "hd"}
    }

    if direction == "right" then
        if self.utils:isPair(coord.y) then
            directionOffsets["right"] = {"bd", "b"}
        else
            directionOffsets["right"] = {"bg"}
        end
    elseif direction == "left" then
        if self.utils:isPair(coord.y) then
            directionOffsets["left"] = {"bd"}
        else
            directionOffsets["left"] = {"bg", "b"}
        end
    elseif direction == "top" then
        if self.utils:isPair(coord.y) then
            directionOffsets["top"] = {"bd"}
        else
            directionOffsets["top"] = {"hd", "d"}
        end
    elseif direction == "bottom" then
        if self.utils:isPair(coord.y) then
            directionOffsets["bottom"] = {"bd", "d"}
        else
            directionOffsets["bottom"] = {"hd"}
        end
    end

    local cellsForDirection = {}
    for _, offset in ipairs(directionOffsets[direction]) do
        if adjacentCells[offset] then
            table.insert(cellsForDirection, adjacentCells[offset])
        end
    end

    return cellsForDirection
end

function mapHelper:isCellOnEdge(cellId, direction)
    local coord = self:cellIdToCoord(cellId)

    if direction == "left" then
        return coord.x == 0 and coord.y < 39
    elseif direction == "right" then
        return coord.x == 13 and coord.y > 0
    elseif direction == "top" then
        return not (coord.x == 0 and coord.y < 2) and not (coord.x == 13 and coord.y == 1) and (coord.y == 0 or coord.y == 1)
    elseif direction == "bottom" then
        return not (coord.x == 13 and coord.y >= 38) and not (coord.x == 0 and coord.y == 38) and (coord.y == 38 or coord.y == 39)
    end

    return false
end

return mapHelper