local pathfinder = {
    dependencies = {"myjson"},
    pathLoaded = false,
    cacheKey = "",
    path = {},
    url = "https://api.v2.dofus-touch.yaya-space.fr/"
}

function pathfinder:moveToWard(goalMapId, goalCellId)
    if tostring(map:currentMapId()) ~= tostring(goalMapId) then
        goalCellId = goalCellId or "-1"
        local cacheKey = goalMapId .. "," .. goalCellId

        if self.cacheKey ~= cacheKey then
            self.pathLoaded = false
            self.cacheKey = cacheKey
        end

        if not self.pathLoaded then
            local function mapWithMostCells(maps)
                local maxCells = 0
                local mapWithMost = nil
                if #maps > 1 then
                    self.logger:warning("Plusieurs maps pour la mapid: " .. goalMapId .. " la carte contenant le + de cellules sera choisi, si la map de destination n'est pas correct veuillez utiliser la fonction avec le paramètre goalCellId, goalCellId correspondant à une cellId marchable de la carte d'arrivé", "Pathfinder")
                end
                for _, map in pairs(maps) do
                    local cellCount = #map.cells -- Compte le nombre de cells dans la table cells
                    if cellCount > maxCells then
                        maxCells = cellCount
                        mapWithMost = map
                    end
                end
                return mapWithMost
            end
            self.logger:info("Chargement du path", "Pathfinder")
            local goodGoalCellId = goalCellId
            local goalMapUuid

            if goalCellId == "-1" then
                local maps = self:getMaps(goalMapId)
                local mapWithMostCells = mapWithMostCells(maps)
                goodGoalCellId = mapWithMostCells.cells[1]
                goalMapUuid = mapWithMostCells.mapUuid
            else
                goalMapUuid = self:getMapUuidByCellAndMapId(goalMapId, goalCellId)
            end

            local currentMapUuid = self:getMapUuidByCellAndMapId(map:currentMapId(), map:currentCell())
            local path = self:getPathTo(currentMapUuid, goalMapUuid)
            if path then
                self.path = path
                self.pathLoaded = true
                self.logger:info("Path chargée", "Pathfinder")
            else
                self.logger:error("Impossible de trouver un path pour la mapid: " .. goalMapId .. " cellid: " .. goalCellId .. ", depuis la mapid: " .. map:currentMapId() .. " cellid: " .. map:currentCell(), "Pathfinder")
            end
        end


        if self.pathLoaded then
            for i, v in pairs(self.path) do
                if v.dir == nil then
                    self.logger:error("Aucune direction pour la map: " .. v.fromMapId, "Pathfinder")
                end
                if (tostring(v.fromMapId) == tostring(map:currentMapId())) then
                    self.logger:info("Changement de carte depuis: (" .. v.fromMapId .. ") vers : (" .. v.toMapId .. ") | direction: " .. v.dir .. " | fromUuid = " .. v.from .. " | toUuid = " .. v.to, "Pathfinder")
                    
                    local sunCell = self:extractNumberFromSunString(v.dir)
                    local doorId = self:extractNumberFromDoorString(v.dir)
                    if sunCell then
                        map:moveToCell(sunCell)
                        global:delay(1000 * 60)
                    elseif doorId then
                        map:useById(doorId, -1)
                        global:delay(1000 * 60)
                    else
                        map:changeMap(v.dir)
                    end
                end

                -- if tostring(v.mapId) == tostring(map:currentMapId()) then
                --     local dir = v.direction
                --     local maps
                --     if i == #self.path then -- Dernière map
                --         maps = self:getMaps(goalMapId)
                --         if #maps == 1 then -- Si la prochaine map est unique, on utilise une cellid de changement de map aléatoire
                --             dir = string.match(v.direction, "(.+)%(")
                --         end
                --     else -- Pendant le trajet
                --         maps = self:getMaps(self.path[i + 1].mapId)
                --         if #maps == 1 then
                --             dir = string.match(v.direction, "(.+)%(")
                --         end
                --     end
                --     self.logger:info("Map: (" .. v.mapId .. "), Change map to: " .. dir, "Pathfinder")
                --     map:changeMap(dir)
                -- end
            end
        end
    else
        self.logger:info("Arrivé sur la map d'arrivé !", "Pathfinder")
    end
end

function pathfinder:moveToWardUuid(goalMapUuid)
    local currentMapUuid = self:getMapUuidByCellAndMapId(map:currentMapId(), map:currentCell())
    --self.logger:log(currentMapUuid)
    if currentMapUuid ~= goalMapUuid then
        local cacheKey = goalMapUuid

        if self.cacheKey ~= cacheKey then
            self.pathLoaded = false
            self.cacheKey = cacheKey
        end

        self.logger:log(currentMapUuid)

        if not self.pathLoaded then
            local path = self:getPathTo(currentMapUuid, goalMapUuid)
            if path then
                self.path = path
                self.pathLoaded = true
                self.logger:info("Path chargée", "Pathfinder")
            else
                self.logger:error("Impossible de trouver un path pour la mapid: " .. goalMapUuid .. ", depuis la mapid: " .. map:currentMapId() .. " cellid: " .. map:currentCell(), "Pathfinder")
            end
        end


        if self.pathLoaded then
            for i, v in pairs(self.path) do
                if v.dir == nil then
                    self.logger:error("Aucune direction pour la map: " .. v.fromMapId, "Pathfinder")
                end
                if (tostring(v.fromMapId) == tostring(map:currentMapId())) then
                    self.logger:info("Changement de carte depuis: (" .. v.fromMapId .. ") vers : (" .. v.toMapId .. ") | direction: " .. v.dir .. " | fromUuid = " .. v.from .. " | toUuid = " .. v.to, "Pathfinder")
                    
                    local sunCell = self:extractNumberFromSunString(v.dir)
                    local doorId = self:extractNumberFromDoorString(v.dir)
                    if sunCell then
                        map:moveToCell(sunCell)
                        global:delay(1000 * 60)
                    elseif doorId then
                        map:useById(doorId, -1)
                        global:delay(1000 * 60)
                    else
                        map:changeMap(v.dir)
                    end
                end

                -- if tostring(v.mapId) == tostring(map:currentMapId()) then
                --     local dir = v.direction
                --     local maps
                --     if i == #self.path then -- Dernière map
                --         maps = self:getMaps(goalMapId)
                --         if #maps == 1 then -- Si la prochaine map est unique, on utilise une cellid de changement de map aléatoire
                --             dir = string.match(v.direction, "(.+)%(")
                --         end
                --     else -- Pendant le trajet
                --         maps = self:getMaps(self.path[i + 1].mapId)
                --         if #maps == 1 then
                --             dir = string.match(v.direction, "(.+)%(")
                --         end
                --     end
                --     self.logger:info("Map: (" .. v.mapId .. "), Change map to: " .. dir, "Pathfinder")
                --     map:changeMap(dir)
                -- end
            end
        end
    else
        self.logger:info("Arrivé sur la map d'arrivé !", "Pathfinder")
    end
end

function pathfinder:getPathTo(startMapUuid, goalMapUuid)
    local response = developer:getRequest(self.url .. "api/Maps/FindShortestPath?fromUuid=" .. startMapUuid .. "&toUuid=" .. goalMapUuid)

    --self.logger:log(response)
    
    if response then
        local decoded = self.myjson:decode(response)
        --self.logger:log(response)
        if decoded.status == 200 then
            return decoded.data
        else
            self.logger:error(decoded.message)
        end
    else
        self.logger:warning("Failed to get path", "Pathfinder")
    end
end

function pathfinder:getMaps(mapId)
    local response = developer:getRequest(self.url .. "api/Maps/GetMapsByMapId?mapId=" .. mapId)
    if response then
        local decoded = self.myjson:decode(response)
        if decoded.status == 200 then
            return decoded.data
        else
            self.logger:error(decoded.message)
        end
    else
        self.logger:warning("Failed to get path", "Pathfinder")
    end
end

function pathfinder:getMapUuidByCellAndMapId(mapId, cellId)
    local response = developer:getRequest(self.url .. "api/Maps/GetMapUuidByCellAndMapId?cell=" .. cellId .. "&mapId=" .. mapId)
    self.logger:log(response)
    if response then
        local decoded = self.myjson:decode(response)
        if decoded.status == 200 then
            return decoded.data.mapUuid
        else
            self.logger:error(decoded.message)
        end
    else
        self.logger:warning("Failed to get path", "Pathfinder")
    end
end

function pathfinder:extractNumberFromSunString(str)
    if str:sub(1, 4) == "sun(" then
        -- Extrait les numéros entre parenthèses
        local number = str:match("%((%d+)%)")
        if number then
            return tonumber(number)
        end
    end
    return nil
end

function pathfinder:extractNumberFromDoorString(str)
    if str:sub(1, 5) == "door(" then
        -- Extrait les numéros entre parenthèses
        local number = str:match("%((%d+)%)")
        if number then
            return tonumber(number)
        end
    end
    return nil
end

return pathfinder